require "active_record"
require "camping"
require "coderay"
require "fileutils"
require "json"
require "markaby"
require "mime/types"
require "pathname"

Camping.goes :Plemp

Markaby::Builder.set(:indent, 2)
unless defined? BASE_DIR
  BASE_DIR = Pathname.new(__FILE__).dirname
  PUBLIC_DIR = BASE_DIR + "public"
  UPLOAD_DIR = BASE_DIR + "upload"
end

module Plemp

  def self.create
    Plemp::Models.create_schema
  end

end

module Plemp::Models

  class Draggable < Base
  end

  class BasicFields < V 1.0
    def self.up
      create_table Draggable.table_name do |d|
        d.string :file
        d.integer :left, :top, :z_index
        # This gives us created_at and updated_at
        d.timestamps
      end
    end
    
    def self.down
      drop_table Draggable.table_name
    end
  end
end

module Plemp::Controllers

  class Index
    def get
      @draggables = Draggable.all
      render :main
    end
  end

  class Current
    def get
      out = Hash.new
      Draggable.all.each { |d| out[d.file] = [d.left, d.top] }
      $stderr.puts("Current status requested: #{out.to_json}")
      out.to_json
    end
  end

  class StaticXX
    def get(type, path)
      mime_type = MIME::Types.type_for(path).first 
      @headers['Content-Type'] = mime_type.nil? ? "text/plain" : mime_type.to_s
      unless path.include? ".." or not ["public", "upload"].include? type
        @headers['X-Sendfile'] = (BASE_DIR + type + path).to_s
      else
        @status = "403"
        "Error 403: Invalid path: #{path}"
      end
    end
  end 

  class SaveposXXX
    def post(file, top, left)
      [top, left].each { |pos| pos.gsub!(/px$/, '') }
      $stderr.puts("Got: file id: #{file} -> top: #{top}, left: #{left}")
      drag = Draggable.find_by_file(file)
      drag.top = top
      drag.left = left
      drag.save
      ""
    end
  end

  class Upload
    def post
      orig_ext = File.extname(@input.file[:filename]).downcase
      file_base = Time.now.strftime("%Y%m%d%H%M%S#{orig_ext}")
      new_file = UPLOAD_DIR + file_base
      new_file.open("w") do |f|
        f.write(@input.file[:tempfile].read)
      end
      Draggable.create(:file => file_base, 
                       :left => 350, :top => 200, :z_index => 0)
      redirect Index
    end
  end

end

module Plemp::Views

  def layout
    xhtml_strict do
      head do
        title "Plemp!"
        link :rel => "stylesheet", :type => "text/css",
             :media => "screen", :href => R(StaticXX, "public", "plemp.css")
        ['prototype', 'scriptaculous', 'dragreg'].each do |js|
          script :src => R(StaticXX, "public", "#{js}.js"), 
                 :type => "text/javascript"
        end
        script :type => "text/javascript" do
          "BaseUrl = \"#{URL()}\";\nsetup_pu()";
        end 
      end
      body do
        self << yield
      end
    end
  end

  def main
    h1 "Plemp!"
    form :action => R(Upload), :method => "post",
         :enctype => "multipart/form-data" do
        input :name => "file", :id => "file", :type => "file"
        input :type => "submit", :value => "Upload!"
    end
    div.draggables! do
      @draggables.each do |d|
        file = UPLOAD_DIR + d.file
        file_type = `file --brief --mime-type #{file}`.chomp
        mime_type = MIME::Types[file_type].first
        if mime_type.nil?
          mime_type = MIME::Type.new(file_type)
        end
        default_style = "left:#{d.left}px;top:#{d.top}px;z-index:0;display:none"
        case mime_type.media_type
          when "image"
            img.draggable :id => d.file, :style => default_style, 
                          :src => R(StaticXX, "upload", file.basename),
                          :alt => file.basename
          when "video"
            # HTML5 is not supported by Markaby!
            self << \
            "    <video class=\"draggable\" id=\"#{d.file}\" " +
              "style=\"#{default_style}\" " +
              "src=\"#{R(StaticXX, "upload", d.file)}\" " + 
              "controls=\"true\">" + "</video>\n"
          when "audio"
            # HTML5 is not supported by Markaby!
            self <<
            "    <audio class=\"draggable\" id=\"#{d.file}\" " +
              "style=\"#{default_style};height=80px;\" " +
              "src=\"#{R(StaticXX, "upload", d.file)}\" " + 
              "controls=\"true\">" + "</audio>\n"
          when "text"
            div.draggable :id => d.file, :style => default_style do
              CodeRay.scan_file(file).div
            end
          else
            span.draggable :id => d.file, :style => default_style do
              em "#{d.file}: Unsupported file type!"
            end
        end
        script(:type => "text/javascript") { "setup_draggable('#{d.file}')" }
      end
    end
  end
end
